// mclbtest.h

#ifndef __MCLBTEST_H
#define __MCLBTEST_H

#include "odlb.h"


/////////////////////////////////////////////////////////////////////////////
//
// This Multi-Column LB test reads a text file "emplist.dat" full of records
// exported from a Paradox database table. It utilizes a helper class,
// HParadoxRec defined in "textrec.h".
//
// New box-class and item-class are derived from HMCListBox and HMCItem to
// accomplish the presentation needs.
//
// The item-class reads the text record, and gives correct strings to display.
// The box-class's Setup() function sets up the column attributes, caption, 
// etc., and processes messages sent from the caption when user click on it.
//
/////////////////////////////////////////////////////////////////////////////
//
// The record format is --
//
// =====================  =================================
//  database table attr.    multi-column LB display attr.
// ---------------------  ---------------------------------
//  #  Type  Field Name    Item Column   Width   Usage
// =====================  =================================
//  0   A                
//  1   A    lastName            3        18%    lastname
//                               2         3%    (space)
//  2   A    firstName           1        30%    firstName
//  3   A    
//  4   A    address             4        40%
//  5   A    
//  6   A    city                5        24%
//  7   A    state               6         8%
//  8   A    postalCode          7        12%
//  9   A    
// 10   A    salutation          0        40%
// 11   A                         
// 12   S    active              *               0/!0: disable/enable
// 13   D    createDate          8        17%
// 14   D    
//
/////////////////////////////////////////////////////////////////////////////

class HMCDemoListbox : public HMCListBox
{
public:
	HMCDemoListbox () { selectedColumn=-1; }
	void Setup (BOOL relative, UINT ctlID=0);
	void FillMyBox ();
	virtual LRESULT MsgProc (UINT, WPARAM, LPARAM);
		// Need to handle more messages -- those from the caption.

protected:
	LRESULT CaptionLUp (WPARAM,LPARAM);
	LRESULT CaptionRUp (WPARAM,LPARAM);

private:
	int selectedColumn;
};


// The following is exactly the same as in MFC or OWL demos --

#include "textrec.h"

class HMyItem : public HMCItem 
{
public:
	HMyItem(HMCDescriptor* des) : HMCItem(des), rec("?AA?A?AAA?A?SD?") {}
	virtual void SetString (LPCSTR str, LONG col=-1) {}
	virtual LPCSTR GetString (LONG col=0) const;
	short GetRec (HParadoxTextRecord&);
protected:
	HParadoxRecord rec;
};


#endif // __MCLBTEST_H
