//	OWNER-DRAW LISTBOX/COMBOBOX CLASS LIBRARY
// COPYRIGHT 1995 JAMES J. HUANG
// ALL RIGHTS RESERVED
//
// WARRANTY: THIS CODE IS PROVIDED AS-IS. THE AUTHOR AND DISTRIBUTORS ARE
//           NOT LIABLE TO ANY DAMAGES THAT MAY INCUR OUT OF USING THIS 
//           CLASS LIBRARY.
//

#ifndef __ODBASE_H
#define __ODBASE_H

// FILE:     ODBASE.H
//
//	CONTENTS: + HELPER FUNCTIONS
//	          + RUN-TIME CLASS INFORMATION MACROS
//           + BOX CLASS AND ITEM CLASS IN GENERAL:
//                class HBaseBox
//                class HListItem
//           + TEXT LISTBOX/COMBOBOX:
//                class HODTextItem
//           + MULTI-COLUMN LISTBOX/COMBOBOX:
//                class HMCDescriptor
//                class HMCLBCaption
//                class HODMCItem
//           + HIERARCHICAL LISTBOX:
//                class HHierBitmapHandler
//	               class HHierItem
//

#ifndef STRICT
#define STRICT
#endif
#include <windows.h>

// Suppliment to Windows facilities --
#ifndef MAKEWORD
#define MAKEWORD(lobyte,hibyte) (((hibyte)<<8)|(lobyte))
#endif
typedef unsigned short USHORT;

extern "C" {
int   NumLines (LPCSTR str);
int   ExtDrawText (HDC hdc, LPCSTR lpsz, RECT FAR* lprc, UINT fuFmt);
		// returns the height of the string.
int   GetTextExtentX (HDC hdc, LPCSTR lpsz, int nsize);	// lpsz may have '\n'.
int   GetTextExtentY (HDC hdc, LPCSTR lpsz, int nsize); // nsize<=0: null-terminated
// Following string functions, pointers may be NULL.
LPSTR _StrCpy (LPSTR,LPCSTR);
BOOL  _StrCmp (LPCSTR,LPCSTR);
int   _StrLen (LPCSTR);
}

// Run-time Info for any class --
class HCLASS_RTI {public: HCLASS_RTI(); int type; };
#define __RT_INFO_BASIC(cls) private: static HCLASS_RTI cls##_hliRTi; \
	public: virtual int Type () const { return cls##_hliRTi.type; } \
			static  int ClassType ()  { return cls##_hliRTi.type; } \
// n-derived class run-time info.
#define RUNTIME_INFO_0(cls) __RT_INFO_BASIC(cls) \
	static  int IsMyType (int tp) { return tp==ClassType(); }
#define RUNTIME_INFO_1(cls,parent) __RT_INFO_BASIC(cls) \
	static  int IsMyType (int tp) { return tp==ClassType()||tp==parent::ClassType(); }
#define RUNTIME_INFO_2(cls,par1,par2) __RT_INFO_BASIC(cls) \
	static  int IsMyType (int tp) { return tp==ClassType()||tp==par1::ClassType()||tp==par2::ClassType(); }
#define RUNTIME_INFO_3(cls,par1,par2,par3) __RT_INFO_BASIC(cls) \
	static  int IsMyType (int tp) { return tp==ClassType()||tp==par1::ClassType()||tp==par2::ClassType()||tp==par3::ClassType(); }
// implementation of class run-time info
#define RUNTIME_INFO_IMPLEMENT(cls) HCLASS_RTI cls::cls##_hliRTi;

#define IS_CLASS(inst,clsname) clsname::IsMyType((inst).Type())
	// TRUE if inst is clsname or one of its derivatives.
#define IS_CLASS_EXACT(inst,clsname) ((inst).Type()==clsname::ClassType())
	// TRUE only if inst is clsname itself
#define IS_DERIVED_FROM(inst,clsname) !IS_CLASS_EXACT(inst,clsname)&&IS_CLASS(inst,clsname)
	// TRUE only if inst is derived from but not clsname.
#define COMPATIBLE_CLASS(x,y) ((x).IsMyType((y).Type())||(y).IsMyType((x).Type()))
	// TRUE if class of x is derived from class of y, or vice versa

//--------------------------------------------------------------------------
// Suppose we have classes A thru E (and corresponding instances, a thru e):
//    A;  B:A;  C:B;  D:B;   E:D;  (draw a diagram of this)
// then, 
// IS_CLASS(e,B) = TRUE
// IS_CLASS(e,A) = TRUE
// IS_CLASS(e,C) = FALSE
// COMPATIBLE_CLASS(d,b) = TRUE
// COMPATIBLE_CLASS(b,d) = TRUE
// COMPATIBLE_CLASS(b,c) = FALSE
//--------------------------------------------------------------------------

#define HLBS_STANDARD (LBS_OWNERDRAWVARIABLE|LBS_SORT|LBS_NOTIFY| \
			LBS_NOINTEGRALHEIGHT|WS_CHILD|WS_HSCROLL|WS_VSCROLL|WS_VISIBLE)
#define HCBS_STANDARD (CBS_OWNERDRAWVARIABLE|CBS_SORT| \
			CBS_NOINTEGRALHEIGHT|WS_CHILD|WS_HSCROLL|WS_VSCROLL|WS_VISIBLE)

#define LEFT    0
#define RIGHT   1
#define CENTER  2
#define OTHER   3
#define MERGE   OTHER
#define TOP     0
#define BOTTOM  1
#define VCENTER 2

typedef BYTE ALIGNMENT;

// ListBox/ComboBox things --
#define LCB_(x)  (isLB?LB_##x:CB_##x)
#define LCBS_(x) (isLB?LBS_##x:CBS_##x)

////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////                                                                    ////
////                     GENERAL OWNER-DRAW CLASSES                     ////
////                                                                    ////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////

		class HListItem;
		class HBaseBox;

////////////////////////////////////////////////////////////////////////////
// The Item Class
////////////////////////////////////////////////////////////////////////////

class HListItem 
{
	friend class HBaseBox;
	
public: // attributes --

	LONG   data;					// general purpose; LB will set/get this value
	USHORT refcount;	// general use -- if refcount>0, dont destroy

public: // methods --

	HListItem(HBaseBox* box, BOOL enable=TRUE);
	virtual ~HListItem() {}
	virtual void SetString (const char*, LONG extra=-1L) =0;
	virtual const char* GetString (LONG extra=-1L) const =0;
	operator const char* () const { return GetString(); }
	void Update () const;	// inline
	BOOL Enable (BOOL en=TRUE);
	BOOL IsEnabled () const { return enabled; }
	virtual short Height () const { return 0; }
		// 0: use default; <0: magnitude = line num
	int Width () const { return width; }
	BOOL IsNull () const;
		
	int  GetIndex () const;		// index in the given listbox/combobox.

protected:      // implementation --

	HBaseBox* theBox;
	
	virtual void draw (const DRAWITEMSTRUCT&) =0;
	virtual void drawFocus (const DRAWITEMSTRUCT&) const; // default: draw focus rect.
	virtual int compare (HListItem* rhs) const	// by default, compares (LPCSTR).
		{ return _StrCmp((LPCSTR)*this,(LPCSTR)*rhs); }
		
	void setWidth (unsigned int newwidth, BOOL refresh=TRUE);
		// sets width, usually in draw();
		// if refresh==TRUE, calls HBaseBox::UpdateHorzExtent().

private:
	USHORT width;		// set only via setWidth()
	USHORT enabled : 1;
	
	RUNTIME_INFO_0(HListItem)
	
};  // HListItem.

////////////////////////////////////////////////////////////////////////////
// The Box Class
////////////////////////////////////////////////////////////////////////////

class HBaseBox 
{
public:		// data

	COLORREF bkColor;
	COLORREF frmColor;
	struct {
		COLORREF norm_text;
		COLORREF gray_text;
		COLORREF sel_text;
		COLORREF norm_bk;
		COLORREF gray_bk;
		COLORREF sel_bk;
		HFONT    font;
	} palette[4];		// general use for text drawing.

	DWORD data;			// box-wide data, general use
	
public:		// functions

	HBaseBox () : sel(this) { initParam(); }
	HWND HWnd () const { return hWnd; }
	BOOL IsLB () const { return isLB; }
	BOOL IsODVariable () const;
	BOOL IsMultiSelect () const;
	int  GetClientWidth () const;
	int  GetClientHeight () const;
	LRESULT NotifyParent (UINT nCode) const;
	LRESULT SendMsg (UINT m,WPARAM wP=0,LPARAM lP=0L) const
		{ return ::SendMessage(hWnd,m,wP,lP); }
	int  GetViewportX () const;
	void Refresh (); // used when conditions change (eg. key column change)
	void Redraw () const { UpdateItem(-1); }

	int  AddItem (HListItem*);
	int  InsertItem (int index, HListItem*);
	HListItem* GetItem (int index) const;
	HListItem* GetCurrentItem () const;
	int  FindItem (int startIndex,LPCSTR,BOOL exact=TRUE) const;
	int  GetItemCount () const { return (int)SendMsg(LCB_(GETCOUNT)); }
	void UpdateItem (int index=-1) const;   // <0: all items
	void DeleteItem (int index) const { SendMsg(LB_DELETESTRING,index); }
	short ItemHeight (int index=-1) const; // index<0: owner-draw fixed
	virtual int ItemWidth (int index) const { return GetItem(index)->Width(); }
	int  EnableItem (int index, int set=TRUE); // return index, or -1.
	BOOL IsItemEnabled (int index) const;
	void MoveTo (int index);
	void UpdateHorzExtent (int newExtent=-1) const;
		// If newExtent <= 0, update to the longest item in the listbox;
		// Otherwise, used that value if > the current extent.
	BOOL SetUpdateHorzExtentOnDelete (BOOL set=TRUE);	// return the last status.
		// if set TRUE, when deleting items, will reset the horizontal extent
		// to the current longest item (listbox default: FALSE).
	int  GetUnit () const { return w_unit; }
	int  Units2Pixels (int x) const { return x * w_unit; }

    // Listbox Selection --
	int  GetSel (int index) const;			// rules out disabled item.
	int  GetSelCount ();					// rules out disabled items.
	int  DeleteSelected (); // returns number of deleted items.
	int  FirstSelected () { return isLB ? sel.First() : -1; }
	int  NextSelected ()  { return isLB ? sel.Next() : -1; }
	int  DeleteDisabled ();

	// helpers --
	void DrawTextWithPalette (int* width, const DRAWITEMSTRUCT&, RECT&, LPCSTR, int palette, 
		ALIGNMENT ha=LEFT, ALIGNMENT va=VCENTER) const;
		// 'width' is the resultant text string width; if it is a multi-line
		// string, it is the longest substring. May be NULL.
	
	class SelectedInHListBox
	{
	public:
		SelectedInHListBox (HBaseBox *lb);
		~SelectedInHListBox ();
		int First ();	// <0 if none.
		int Next ();	// <0 if none.
		int Current();	// <0 if none.
	protected:
		HBaseBox* theBox;
		int  count;
		int  pointer;
		int  *ar;
	};

protected:      // data

	// Init'ed by furtherInit() --
	HWND hWnd;
	UINT id;	// control id.
	USHORT isLB : 1;  // 1/0 for listbox/combobox.
	USHORT itemHeight;
	USHORT w_unit;	// = average font char width
	USHORT updateHExtOnDelete : 1;

protected:      // functions

	void furtherInit (HWND hwnd, BOOL isL);	// MUST be called when hwnd becomes valid

	// echoed message handlers --
	int  compare_ (COMPAREITEMSTRUCT &cis) const
		{ return ((HListItem*)cis.itemData1)->compare((HListItem*)cis.itemData2); }
	void measure_ (MEASUREITEMSTRUCT &mis) const
		{ mis.itemHeight = ItemHeight(IsODVariable() ? mis.itemID : -1); }
	void delete_  (DELETEITEMSTRUCT &)  const;
	void draw_    (DRAWITEMSTRUCT &)    const;

	// fix the PgDn bug for Windows 3.1 OD-variable listboxes --
#ifndef WIN32
	void fixPgDnVODLB ();
#endif

private:	
	void initParam ();
	SelectedInHListBox sel;

	RUNTIME_INFO_0(HBaseBox)

};  // HBaseBox.

// inline function --
inline void HListItem::Update () const { theBox->UpdateItem(GetIndex()); }


////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////                                                                    ////
////                       TEXT OWNER-DRAW CLASSES                      ////
////                                                                    ////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////

#include "hstring.h"

class HTextItem : public HListItem 
{
public:
	USHORT align;
	USHORT palette;
	USHORT leftOffset;

	HTextItem (HBaseBox* box, LPCSTR str=NULL, BOOL en=1, 
				ALIGNMENT _align=LEFT, unsigned _pal=0, unsigned _lOfst=0);
	virtual short Height () const { return -NumLines((const char*)(*this)); }
	virtual void SetString (const char* s, LONG extra=-1L) { str=s; }
	virtual const char* GetString (LONG extra=-1L) const { return (const char*)str; }

protected:
	HString str;
	virtual void draw (const DRAWITEMSTRUCT&);
	
	RUNTIME_INFO_1(HTextItem,HListItem)

};	// HTextItem.


////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////                                                                    ////
////                  MULTI-COLUMN OWNER-DRAW CLASSES                   ////
////                                                                    ////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////

#include "mstring.h"

// MultiColumnListBox Caption Mouse message --
#define MCLB_CAPTIONLDOWN		WM_USER+100	// watch out, LB_XXXX's also use WM_USER!
#define MCLB_CAPTIONLUP			WM_USER+101
#define MCLB_CAPTIONLDBLCLK		WM_USER+102
#define MCLB_CAPTIONRDOWN		WM_USER+103
#define MCLB_CAPTIONRUP			WM_USER+104
#define MCLB_CAPTIONRDBLCLK		WM_USER+105
	// When the caption gets such clicked, these messages are sent to the
	// listbox; wParam contains the key flags, and lParam, column #.
	// In case a column is of "other" alignment, the returned column number 
	// is the immediate rightmost one that is not "other".

		class HColItem;
		class HMCCaption;
		class HMCDescriptor;
		class HMCItem;


struct HColAttr {
	unsigned int horzAlign : 2;
	unsigned int vertAlign : 2;
	unsigned int palette   : 2; // up to 4 palettes in HBaseBox.
	unsigned int horzGrid  : 1;
	unsigned int vertGrid  : 1;
	
	HColAttr () { *(char*)this = (char)0; }
	HColAttr (unsigned int hAlign, unsigned int vAlign, unsigned int pal, 
				unsigned int hGrid, unsigned int vGrid);
};


class HColItem 
{
public:
	HColItem () { position=0; }
	HColAttr     attr;
	unsigned int position;  // the right boundary of that column.

};  // HColItem.


class HMCLBCaption
{
	friend class HMCDescriptor;

public: // data
	COLORREF colorText;
	COLORREF colorBkgnd;
	COLORREF colorFrame;
	BOOL     hasFrame;

	HMultiString ms;

public:	// functions
	HMCLBCaption (HMCDescriptor* mcd);
	~HMCLBCaption ();
	void SetTabStops (int numTabs, WORD *tabs=NULL);
		// if tabs NULL, then by default set all alignments to center.
		// 1) tabs[]'s lo-byte is alignment; it may have 'other', meaning 
		// this column is merged into the immediate right one, use its 
		// alignment and text for the merged one.
		// 2) tabs[]'s hi-byte is the left offset, if it is left-aligned.
	void Move ();
	void SetDistance (short newDist=3) { distance=newDist; }
	void SetHeight (short newHeight) { height=newHeight; }
	void Create (short newHeight, short newDist=3);
	void Update () const;
	short Height () const { return height; }
	int  HeightAndDistance () const { return height+distance; }
	void ResetPos () { viewportX=0; }

protected:
	HMCDescriptor*  descriptor;
	ALIGNMENT*		alignments;		// same length as ms.NumStrings().
	BYTE*           leftOffsets;	// left offset for left-alignment (only).
	
	HWND     hWnd;
	short    height;				// in pixels
	short    distance;				// number of pixels above the listbox.
	int      viewportX;				// updated during drawing; 
									// may be used for position test.
	int      columnNumber () const { return ms.NumStrings(); }
	int      getViewportX();
	virtual void draw (HDC);
	void     getRect (LPRECT);		// returns rect in the parent window.
	int      xToColumn (int x) const;	// turns mouse x-pos into column #.
	HBaseBox* theBox () const; // inline
	
private:
	void initParam ();

	friend LRESULT CALLBACK CaptionWndProc (HWND,UINT,WPARAM,LPARAM);
	friend void PASCAL caption_drawing (HWND,HDC);
	
	RUNTIME_INFO_0(HMCLBCaption)

};  // HMCLBCaption.


class HMCDescriptor
{
	friend class HMCItem;
	friend class HMCLBCaption;

public:

	HMCDescriptor (HBaseBox*);
	~HMCDescriptor ();

	// Columns setup -- 
	void SetTabstopsAbsolute (int numTabs,WORD *tabs,HColAttr* a=NULL,int k1=0,int k2=-1,int k3=-1);
	void SetTabstopsUnits (int numTabs,WORD *tabs,HColAttr* a=NULL,int k1=0,int k2=-1,int k3=-1);
		// numTabs is in terms of w_unit.
	void SetTabsPercent (BOOL relative,int numTabs,WORD *tabs,HColAttr* a=NULL,int k1=0,int k2=-1,int k3=-1);
		// In terms of the width of the listbox at run-time if relative is TRUE, or 
		// creation time if relative is FALSE.
	void UpdateHorzExtent () const;
		
	int  NumColumns () const { return numCols; }
	HColItem& GetColItem (int col) const;
	int  ColRightBound(int) const;
	int  RightBound () const { return ColRightBound(numCols-1); }
	void UpdateItem (int index,int col) const; // index/col<0: all items/col's.
	void UseHorzGridline (BOOL yes=1) { bHorzGridline=yes; }
		// if not, diff fields may still have gridlines; 
		// if yes, a horizontal grid line is drawn for each item anyway.

	LPCSTR GetString (int index,int col=-1) const;
	int  SetString (LPCSTR, int index,int col=-1, BOOL refresh=TRUE);
	int  GetStringLen (int index,int col=-1) const { return _StrLen(GetString(index,col)); }
	int  CompareString (LPCSTR str,int index,int col=-1) const;
	int  FindString (LPCSTR str,int Index=0,int col=-1,BOOL exact=0) const;
	LPCSTR GetSelString (int col=-1);
	int  SetSelString (LPCSTR str,int col=-1);

	// Caption methods --
	HMCLBCaption* Caption () { return caption; }
	void SetCaption (HMCLBCaption* caption, BOOL _ownsCaption=TRUE);
	void SetCaption (short numTabs, WORD *tabs=NULL)
			{ SetCaption(new HMCLBCaption(this)); SetCaptionStops(numTabs,tabs); }
	void SetCaptionStops (short numTabs, WORD *tabs=NULL)
			{ if (HasCaption()) caption->SetTabStops(numTabs,tabs); }
	BOOL HasCaption () const { return (caption!=NULL); }
    void MoveCaptionBox (LPRECT rc, BOOL repaint=TRUE);
		// Move the Caption and the Listbox as a whole in 'rc'.
	void MoveCaption (short newHeight, short newDist=3);
	void SetCaptionDistance (short newDist=3)
			{ if (HasCaption()) caption->SetDistance(newDist); }
	void CreateCaption (short newHeight, short newDist=3)
			{ if (HasCaption()) caption->Create(newHeight,newDist); }
	void UpdateCaption () const { if (HasCaption()) caption->Update(); }
	void ResetCaption () { if (HasCaption()) { caption->ResetPos(); caption->Update(); } }

	short KeyCol[3];

protected:

	HBaseBox* theBox;
	HMCLBCaption* caption;

	// Set by SetTabStops() --
	HColItem *column;
	short numCols;
	USHORT bHorzGridline : 1;
	USHORT bRelativeColumns : 1;

private:
	USHORT bOwnsCaption : 1;
	
	RUNTIME_INFO_0(HMCDescriptor)

};	// HMCDescriptor.                


class HMCItem : public HListItem 
{
public:
	HMCItem(HMCDescriptor* des, BOOL enable=1);

	virtual void SetString (LPCSTR str, LONG col=-1L) =0;
	void SetString (int col, LPCSTR str) { SetString(str,(LONG)col); }
	virtual LPCSTR GetString (LONG col=-1L) const =0;
	int CompareString (LPCSTR str, int col) const { return _StrCmp(GetString(col),str); }
	virtual short Height () const;
	HMCDescriptor* GetDescriptor() const { return (HMCDescriptor*)theBox->data; }
	int  NumColumns () const { return GetDescriptor()->NumColumns(); }

protected:

	virtual int compare(HListItem* y) const;
	virtual void draw (const DRAWITEMSTRUCT &dis);
	virtual void drawFocus (const DRAWITEMSTRUCT&) const;
	virtual void drawField (const DRAWITEMSTRUCT&,int col) const;
	void getFieldRect(const RECT& itemRect,RECT& fieldRect,int col) const;
	
	RUNTIME_INFO_1(HMCItem,HListItem)
	
};  // HMCItem.

// inline functions --
inline HBaseBox* HMCLBCaption::theBox () const { return descriptor->theBox; }


////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////                                                                    ////
////                  HIERARCHICAL OWNER-DRAW CLASSES                   ////
////                                                                    ////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////

struct HBitmapInfo
{
	HBITMAP hBitmap;
	short   width;
	short   height;

	HBitmapInfo ();
	~HBitmapInfo ();
	void LoadResource (HINSTANCE,UINT resId);
};


class HHierBitmapHandler
{
	friend class HHierItem;
public:
	HBaseBox* theBox;
	USHORT threadWidth;
	
	HHierBitmapHandler (HBaseBox*, HINSTANCE hinst, USHORT bitmaps, 
						UINT bitmapIDs[]);
		// By default, HHierItem uses 
		// bitmapNames[0] for terminal node,
		// bitmapNames[1] for closed non-terminal node,
		// bitmapNames[2] for opened non-terminal node, and
		// bitmapNames[1] for non-terminal node with no children.
		// Provide more bitmap names for HHierItem to have more icons to display.
	~HHierBitmapHandler ();
	virtual void Draw (short iconpalette, HDC, short startX, short startY) const;
	short  Height (USHORT iconpalette) const;
	short  Width  (USHORT iconpalette) const;
	short  TextOffset () const { return textOffset; }
	HBITMAP GetHBitmap (USHORT iconpalette) const;
protected:
	USHORT nPalettes;
	USHORT textOffset;	// = the max. icon width + 8.
	HBitmapInfo* bitmaps;
};

 
// class HHierItem : public HListItem
//
// Hierarchical structure implementation --
//
//  Each item has a pointer to its first child, and a pointer to its next 
//  sibling. Also has a flag indicating if it is the last of the sibling; 
//  if so the case, the sibling pointer points (back) to the parent.
//  If no children, the child pointer points (back) to itself.
//
// Because of (thanks to) this tree structure, you should not insert or
// delete items directly to/from the listbox; instead, manipulate them 
// through the tree structure methods. May call refresh() to reflect the 
// change of structure in the listbox.
//

	class HHierItem;
	class HHierItemTraverser {public: virtual BOOL Action(HHierItem*)=0; };
		// Action(): TRUE/FALSE to continue/stop.

class HHierItem : public HListItem
{
public:
	// Standard HListItem functions --
	HHierItem (HHierBitmapHandler*,LPCSTR str,BOOL isterminal,BOOL en=TRUE,
		short pal_text=0,short pal_unopenable=-1, short pal_closed=-1, short pal_opened=-1);
		// if isterminal, pal_unopenable is the index of the icon palette 
		// (default 0); if not, pal_unopenable is the index of the icon 
		// palette if no children (default 1);
		// pal_closed is the index of the icon palette when closed (default 1);
		// pal_opened is the index of the icon palette when opened (default 2).
	virtual const char* GetString (LONG extra=-1L) const { return (LPCSTR)text; }
	virtual void SetString (const char* str,LONG extra=-1L) { text=str; }
	virtual short Height () const { return 4 + BMH()->Height(curPalette()); }
	BOOL LoadTreeToListBox (HBaseBox*) const;	// Add the root item to the LB
	BOOL IsLoaded () const { return theBox->GetItem(0)==GetRoot(); }

	// Hierarchical item manipulation --
	BOOL AddChildToFirst (HHierItem*);
	BOOL AddChildToLast  (HHierItem*);
	BOOL AddChildSorted  (HHierItem*, BOOL ascending=TRUE);
		// either always call this or never; sorted by text.
	HHierItem* GetRoot () const;
	HHierItem* GetParent () const;
	HHierItem* GetFirstChild () const  { return child; }
	HHierItem* GetPrevSibling () const;	// if first child, return NULL.
	HHierItem* GetNextSibling () const  { return lastSibling?NULL:sibling; }
	BOOL Traverse (HHierItemTraverser* hit);
		// If TRUE, continue; otherwise, stop.
		// For each item x, if (hit->action(x)) continue; else return;
		// Derive from HHierItemTraverser for various traversal tasks.
	BOOL IsRoot () const  { return (sibling==this); }
	BOOL HasChild () const  { return (child!=this); }
	BOOL IsLastSibling () const { return lastSibling; }
	int  NumChildren () const;
	BOOL IsDescendantOf (HHierItem*) const;
	int  Depth () const;		// root is 0.
	BOOL IsTerminal () const { return isTerminal; }
	BOOL IsOpen () const  { return !isTerminal && isOpen; }
	void SetOpen (BOOL open=TRUE) { if (!isTerminal) isOpen = open ? 1 : 0; }
	// -- For the following pairs of APIs, 'BOOL loaded' is used to improve performance.
	void DestroyAll (BOOL loaded);	// everyone is supposed 'new'ed!
	void DestroyAll () { DestroyAll(IsLoaded()); }
	void DetachSubtree (BOOL loaded);	// this node becomes a new root.
	void DetachSubtree () { DetachSubtree(IsLoaded()); }
	void DoOpen (BOOL loaded);
	void DoOpen () { DoOpen(IsLoaded()); }
	void DoClose (BOOL loaded) { _doClose(loaded); theBox->UpdateHorzExtent(); }
	void DoClose () { DoClose(IsLoaded()); }
	void DoOpenClose (BOOL loaded) { if (IsOpen()) DoClose(loaded); else DoOpen(loaded); }
	void DoOpenClose () { DoOpenClose(IsLoaded()); }
	
protected:	// listbox item attributes --
	HString text;
	USHORT isTerminal : 1;
	USHORT isOpen     : 1;	// if !isTerminal && has child(ren)
	USHORT textPalette;
	
	USHORT palette4opened;		// if !isTerminal, the icon for opened status.
	USHORT palette4closed;		// if !isTerminal, the icon for closed status.
	USHORT palette4unopenable;	// if isTerminal, the icon for the terminal node.
			// if !isTerminal, the icon for having no children;
			// -- default is the same as palette4closed.

    HHierBitmapHandler* BMH () const { return (HHierBitmapHandler*)theBox->data; }
	USHORT curPalette () const;
	USHORT iconWidth () const { return BMH()->Width(curPalette()); }
	
	virtual void draw (const DRAWITEMSTRUCT&);
	virtual void drawFocus (const DRAWITEMSTRUCT&) const {}
	virtual int  compare (HListItem*) const { return 1; }	// disable it.

protected:	// tree structure attrubites --
	HHierItem* child;		// points to the first child. if no child, points to self.
	HHierItem* sibling;		// points to the next sibling, or to parent if 
							// lastSibling is TRUE, or self if is root.
	USHORT lastSibling : 1; // Also used for drawing.

	RUNTIME_INFO_1(HHierItem,HListItem)

private:
	void _processLBAfterAdd ();		// after an item is added.
	void _doClose (BOOL loaded);	// recursive, called by DoClose().
	
};	// HHierItem.


#ifdef WIN32	// WIN16/32 COMPATIBLE STUFF ///////////////////////////
#ifndef EXPORT
#define EXPORT
#endif
#define GETINSTANCE(hw) (HINSTANCE)::GetWindowLong(hw,GWL_HINSTANCE)
#define _MoveTo(hdc,x,y) MoveToEx(hdc,x,y,NULL)
#define _SetViewportOrg(hdc,x,y) SetViewportOrgEx(hdc,x,y,NULL)
#else	// !WIN32 //////////////////////////////////////////////////////
#ifndef EXPORT
#define EXPORT _export
#endif
#define GETINSTANCE(hw) (HINSTANCE)::GetWindowWord(hw,GWW_HINSTANCE)
#define _MoveTo(hdc,x,y) MoveTo(hdc,x,y)
#define _SetViewportOrg(hdc,x,y) SetViewportOrg(hdc,x,y)
#endif	////////////////////////////////////////////////////////////////

#endif // __ODBOX_H
