// OWNER-DRAW LISTBOX/COMBOBOX CLASS LIBRARY
// COPYRIGHT 1995 JAMES J. HUANG
// ALL RIGHTS RESERVED
//
// WARRANTY: THIS CODE IS PROVIDED AS-IS. THE AUTHOR AND DISTRIBUTORS ARE
//           NOT LIABLE TO ANY DAMAGES THAT MAY INCUR OUT OF USING THIS 
//           CLASS LIBRARY.
//

#ifndef __ODCB_H
#define __ODCB_H

// FILE:     ODCB.H
//
//      CONTENTS: COMBOBOX CLASS IN GENERAL:
//           FOR MFC2:        class CHComboBox
//           FOR OWL2:        class THComboBox
//           FOR INDEPENDENT: class HComboBox
//

#if !defined(__MFC) && !defined(__OWL) && !defined(__IND)
Stop!!!  Must defined __MFC, __OWL or __IND!
#endif

#ifdef __MFC
#ifndef WIN32
#include "stdafx.h"
#else   // WIN32
#include <afxwin.h>
#endif
#endif
#ifdef __OWL
Please comment out this and uncomment the next; Visual C++ doesnot tolerate this!
//#include <owl\combobox.h>
#endif

#include "odbase.h"

////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////                                                                    ////
////                GENERAL OWNER-DRAW COMBOBOX CLASSES                 ////
////                                                                    ////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////

#if defined(__MFC) || defined(__OWL)
#ifdef __MFC
class CHComboBox : public CComboBox, public HBaseBox 
#else // __OWL
class THComboBox : public TComboBox, public HBaseBox 
#endif
{
public:

// --------------------------------------------------------------------------
// Constructors, create/start and other common features --
// --------------------------------------------------------------------------
    
#ifdef __MFC
	CHComboBox () { }
	BOOL Create(DWORD dwStyle,const RECT& rect,CWnd* pParentWnd,UINT nID);
	void SubclassDlgItem (UINT nID,CWnd* pParent);
	BOOL SubclassWindow (HWND hWnd);
#else // OWL
	THComboBox (TWindow *Parent,int Id,int x,int y,int w,int h,DWORD style,
				int textLen,TModule *M=0);
	THComboBox (TWindow *Parent,int ResourceId,int textLen=0,TModule *M=0)
		: TComboBox(Parent,ResourceId,textLen,M) { }
	void SetupWindow ();
#endif

// --------------------------------------------------------------------------
// Affected CComboBox/TComboBox public member functions --
// --------------------------------------------------------------------------

	// string-related methods --
	int  SetItemData (int index,DWORD itemData);
	DWORD GetItemData (int index) const { return GetItem(index)->data; }
	int  GetItemHeight(int index) { return ItemHeight(index); }
	int  GetSel (int index) const { return HBaseBox::GetSel(index); }
#ifdef __MFC
	int  GetText (int index,LPSTR str) const;
	void GetText (int index,HString&) const;
	void GetText (int index,CString&) const;
	int  GetTextLen (int index) const;
	int  FindString (int startAfter,LPCSTR str) { return FindItem(startAfter,str,FALSE); }
	int  FindStringExact (int startAfter,LPCSTR str) { return FindItem(startAfter,str,TRUE); }
	int  SelectString (int startAfter,LPCSTR str);
	void* GetItemDataPtr (int index) const { return (void*)GetItemData(index); }
	int  SetItemDataPtr (int index,void* pdata) { return SetItemData(index,(DWORD)pdata); }
#else // OWL
	int  GetString (LPSTR str,int index) const;
	void GetString (HString& s,int index) const;
	int  GetStringLen (int index) const;
	int  FindString (LPCSTR str,int schIdx) { return FindItem(schIdx,str,FALSE); }
	int  FindExactString (LPCSTR str,int schIdx) { return FindItem(schIdx,str,TRUE); }
	int  GetSelString (LPSTR str,int maxChar) const;
	int  SetSelString (LPCSTR str,int schIdx);
#endif

// --------------------------------------------------------------------------
//  Suppressed C/TComboBox public member functions --
// --------------------------------------------------------------------------

	int  AddString (LPCSTR str) { return -1; }
	void SetColumnWidth (int width) { }
#ifdef __MFC
	int  InsertString (int Index,LPCSTR str) { return -1; }
#else // OWL
	int  InsertString (LPCSTR str,int Index) { return -1; }
	int  SetItemHeight (int index,int height) { return -1; }
#endif

// --------------------------------------------------------------------------
//  Implementation --
// --------------------------------------------------------------------------

protected:

	// Message handling --
#ifdef __MFC
	virtual void DrawItem (DRAWITEMSTRUCT *di) { draw_(*di); }
	virtual void MeasureItem (MEASUREITEMSTRUCT *mi) { measure_(*mi); }
	virtual void DeleteItem (DELETEITEMSTRUCT *di) { delete_(*di); }
	virtual int  CompareItem (COMPAREITEMSTRUCT *ci) { return compare_(*ci); }
	afx_msg BOOL OnEraseBkgnd(CDC*);
	DECLARE_MESSAGE_MAP();
#else // OWL
	virtual void DrawItem (DRAWITEMSTRUCT &di) { draw_(di); }
	virtual void MeasureItem (MEASUREITEMSTRUCT &mi) { measure_(mi); }
	virtual void DeleteItem (DELETEITEMSTRUCT &di) { delete_(di); }
	virtual int  CompareItem (COMPAREITEMSTRUCT &ci) { return compare_(ci); }
#endif

#ifdef __MFC
	RUNTIME_INFO_1(CHComboBox,HBaseBox)
#else
	RUNTIME_INFO_1(THComboBox,HBaseBox)
#endif

};
#endif  // __MFC || __OWL


#ifdef __IND
class HComboBox : public HBaseBox
{
public:
	HComboBox ();
	~HComboBox ();
	BOOL Create(DWORD dwStyle,const RECT& rect,HWND hParent);
	BOOL SubclassDlgItem (HWND hDlg, UINT id)
		{ return SubclassBox (::GetDlgItem(hDlg,id)); }
	BOOL SubclassBox (HWND hwnd);
	BOOL UnsubclassBox ();
	virtual LONG MsgProc (UINT, WPARAM, LPARAM);
		// Specific listbox messages are processed by default;
		// You may override this to do custome message processing.
		// In addition, the parent should call this function for WM_DRAWITEM, 
		// WM_MEASUREITEM, WM_DELETEITEM and WM_COMPAREITEM.

protected:
	FARPROC lpfnOrigProc;
	FARPROC lpfnMyProc;
	RUNTIME_INFO_1(HComboBox,HBaseBox)
};
#endif  // __IND

#endif  // __ODCB_H
