// OWNER-DRAW LISTBOX/COMBOBOX CLASS LIBRARY
// COPYRIGHT 1995 JAMES J. HUANG
// ALL RIGHTS RESERVED
//
// WARRANTY: THIS CODE IS PROVIDED AS-IS. THE AUTHOR AND DISTRIBUTORS ARE
//           NOT LIABLE TO ANY DAMAGES THAT MAY INCUR OUT OF USING THIS 
//           CLASS LIBRARY.
//

#ifndef __ODLB_H
#define __ODLB_H

// FILE:     ODLB.H
//
//      CONTENTS: + LISTBOX CLASS IN GENERAL:
//             FOR MFC2:        class CHListBox
//             FOR OWL2:        class THListBox
//             FOR INDEPENDENT: class HListBox
//           + MULTI-COLUMN LISTBOX:
//             FOR MFC2:        class CHMCListBox
//             FOR OWL2:        class THMCListBox
//             FOR INDEPENDENT: class HMCListBox
//

#if !defined(__MFC) && !defined(__OWL) && !defined(__IND)
Stop!!!  Must defined __MFC, __OWL or __IND!
#endif

#ifdef __MFC
#ifndef WIN32
#include "stdafx.h"
#else   // WIN32
#include <afxwin.h>
#endif
#endif
#ifdef __OWL
Please comment out this and uncomment the next; Visual C++ doesnot tolerate this!
//#include <owl\listbox.h>
#endif

#include "odbase.h"

////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////                                                                    ////
////                GENERAL OWNER-DRAW LISTBOX CLASSES                  ////
////                                                                    ////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////

#if defined(__MFC) || defined(__OWL)
#ifdef __MFC
class CHListBox : public CListBox, public HBaseBox 
#else // __OWL
class THListBox : public TListBox, public HBaseBox 
#endif
{
public:

// --------------------------------------------------------------------------
// Constructors, create/start and other common features --
// --------------------------------------------------------------------------
    
#ifdef __MFC
	CHListBox () { }
	BOOL Create(DWORD dwStyle,const RECT& rect,CWnd* pParentWnd,UINT nID);
	void SubclassDlgItem (UINT nID,CWnd* pParent);
	BOOL SubclassWindow (HWND hWnd);
#else // OWL
	THListBox (TWindow *Parent,int Id,int x,int y,int w,int h,TModule *M=0);
	THListBox (TWindow *Parent,int ResourceId,TModule *M=0)
		: TListBox(Parent,ResourceId,M) { }
	void SetupWindow ();
#endif

// --------------------------------------------------------------------------
// Affected CListBox/TListBox public member functions --
// --------------------------------------------------------------------------

	// string-related methods --
	int  SetItemData (int index,DWORD itemData);
	DWORD GetItemData (int index) const { return GetItem(index)->data; }
	int  GetItemHeight(int index) { return ItemHeight(index); }
	int  GetSel (int index) const { return HBaseBox::GetSel(index); }
	int  GetSelCount () { return HBaseBox::GetSelCount(); }
	void SetHorizontalExtent (int hExt) { UpdateHorzExtent(hExt); }
#ifdef __MFC
	int  GetText (int index,LPSTR str) const;
	void GetText (int index,HString&) const;
	void GetText (int index,CString&) const;
	int  GetTextLen (int index) const;
	int  FindString (int startAfter,LPCSTR str) { return FindItem(startAfter,str,FALSE); }
	int  FindStringExact (int startAfter,LPCSTR str) { return FindItem(startAfter,str,TRUE); }
	int  SelectString (int startAfter,LPCSTR str);
	void* GetItemDataPtr (int index) const { return (void*)GetItemData(index); }
	int  SetItemDataPtr (int index,void* pdata) { return SetItemData(index,(DWORD)pdata); }
#else // OWL
	int  GetString (LPSTR str,int index) const;
	void GetString (HString& s,int index) const;
	int  GetStringLen (int index) const;
	int  FindString (LPCSTR str,int schIdx) { return FindItem(schIdx,str,FALSE); }
	int  FindExactString (LPCSTR str,int schIdx) { return FindItem(schIdx,str,TRUE); }
	int  GetSelString (LPSTR str,int maxChar) const;
	int  GetSelStrings (LPSTR* strs,int maxCnt,int maxChar) const;
	int  SetSelString (LPCSTR str,int schIdx);
	int  SetSelStrings (LPCSTR* strs,int numSels,BOOL shouldSet);
#endif

// --------------------------------------------------------------------------
//  Suppressed C/TListBox public member functions --
// --------------------------------------------------------------------------

	int  AddString (LPCSTR str) { return -1; }
	void SetColumnWidth (int width) { }
#ifdef __MFC
	int  InsertString (int Index,LPCSTR str) { return -1; }
	int Dir (UINT attr,LPCSTR wildCard) { return -1; }
	void SetTabStops () { }
	void SetTabStops (int numTabs,int *tabs) { }
	BOOL SetTabStops (const int& eachStop) { return FALSE; }
#else // OWL
	int  InsertString (LPCSTR str,int Index) { return -1; }
	int  DirectoryList (UINT attrs,LPCSTR fileSpec) { return -1; }
	int  SetItemHeight (int index,int height) { return -1; }
	void SetTabStops (int numTabs=0,int *tabs=NULL) { }
#endif

// --------------------------------------------------------------------------
//  Implementation --
// --------------------------------------------------------------------------

protected:

	// Message handling --
#ifdef __MFC
	virtual void DrawItem (DRAWITEMSTRUCT *di) { draw_(*di); }
	virtual void MeasureItem (MEASUREITEMSTRUCT *mi) { measure_(*mi); }
	virtual void DeleteItem (DELETEITEMSTRUCT *di) { delete_(*di); }
	virtual int  CompareItem (COMPAREITEMSTRUCT *ci) { return compare_(*ci); }
#ifndef WIN32   // to fix the PgDn bug for Win3.1 OD Variable LB.
	afx_msg void OnKeyDown (UINT,UINT,UINT);
#endif
	afx_msg void OnChar (UINT,UINT,UINT);
	afx_msg BOOL OnEraseBkgnd(CDC*);
	DECLARE_MESSAGE_MAP();
#else // OWL
	virtual void DrawItem (DRAWITEMSTRUCT &di) { draw_(di); }
	virtual void MeasureItem (MEASUREITEMSTRUCT &mi) { measure_(mi); }
	virtual void DeleteItem (DELETEITEMSTRUCT &di) { delete_(di); }
	virtual int  CompareItem (COMPAREITEMSTRUCT &ci) { return compare_(ci); }
#ifndef WIN32   // to fix the PgDn bug for Win3.1 OD Variable LB.
	void EvKeyDown (UINT,UINT,UINT);
#endif
	void EvChar (UINT,UINT,UINT);
	DECLARE_RESPONSE_TABLE(THListBox);
#endif

#ifdef __MFC
	RUNTIME_INFO_1(CHListBox,HBaseBox)
#else
	RUNTIME_INFO_1(THListBox,HBaseBox)
#endif

};
#endif  // __MFC || __OWL


#ifdef __IND
class HListBox : public HBaseBox
{
public:
	HListBox ();
	~HListBox ();
	BOOL Create(DWORD dwStyle,const RECT& rect,HWND hParent);
	BOOL SubclassDlgItem (HWND hDlg, UINT id)
		{ return SubclassBox (::GetDlgItem(hDlg,id)); }
	BOOL SubclassBox (HWND hwnd);
	BOOL UnsubclassBox ();
	virtual LONG MsgProc (UINT, WPARAM, LPARAM);
		// Specific listbox messages are processed by default;
		// You may override this to do custome message processing.
		// In addition, the parent should call this function for WM_DRAWITEM, 
		// WM_MEASUREITEM, WM_DELETEITEM and WM_COMPAREITEM.

protected:
	FARPROC lpfnOrigProc;
	FARPROC lpfnMyProc;
	RUNTIME_INFO_1(HListBox,HBaseBox)
};
#endif  // __IND


////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////                                                                    ////
////              MULTI-COLUMN OWNER-DRAW LISTBOX CLASSES               ////
////                                                                    ////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////

#ifdef __MFC
class CHMCListBox : public CHListBox, public HMCDescriptor
{
public:
	CHMCListBox () : HMCDescriptor((HBaseBox*)this) { }
	void SetTabStops (int numTabs,int *tabs);
	void Refresh () { HBaseBox::Refresh(); ResetCaption(); }
		
protected:
	afx_msg void OnHScroll (UINT,UINT,CScrollBar*);
	afx_msg void OnSize (UINT,int,int);
	DECLARE_MESSAGE_MAP()
	RUNTIME_INFO_1(CHMCListBox,CHListBox)
};
#endif  // __MFC

#ifdef __OWL
class THMCListBox : public THListBox, public HMCDescriptor
{
public:
	THMCListBox (TWindow *Parent,int Id,int x,int y,int w,int h,TModule *M=0)
		: THListBox(Parent,Id,x,y,w,h,M), HMCDescriptor((HBaseBox*)this) { }
	THMCListBox (TWindow *Parent,int ResourceId,TModule *M=0)
		: THListBox(Parent,ResourceId,M), HMCDescriptor((HBaseBox*)this) { }
	void SetTabStops (int numTabs=0,int *tabs=NULL);
	void Refresh () { HBaseBox::Refresh(); ResetCaption(); }

protected:
	void EvHScroll (UINT,UINT,HWND);
	void EvSize (UINT,TSize&);
	DECLARE_RESPONSE_TABLE(THMCListBox);
	RUNTIME_INFO_1(THMCListBox,THListBox)
};
#endif  // __OWL

#ifdef __IND
class HMCListBox : public HListBox, public HMCDescriptor
{
public:
	HMCListBox () : HMCDescriptor((HBaseBox*)this) { }
	void Refresh () { HBaseBox::Refresh(); ResetCaption(); }
	virtual LRESULT MsgProc (UINT, WPARAM, LPARAM);
		// further processes WM_HSCROLL and WM_SIZE.
	RUNTIME_INFO_1(HMCListBox,HListBox)
};
#endif  // __IND

#endif  // __ODLB_H

