//	OWNER-DRAW LISTBOX/COMBOBOX CLASS LIBRARY
// COPYRIGHT 1995 JAMES J. HUANG
// ALL RIGHTS RESERVED
//
// WARRANTY: THIS CODE IS PROVIDED AS-IS. THE AUTHOR AND DISTRIBUTORS ARE
//           NOT LIABLE TO ANY DAMAGES THAT MAY INCUR OUT OF USING THIS 
//           CLASS LIBRARY.
//

#ifndef __TEXTREC_H
#define __TEXTREC_H

// FILE:     TEXTREC.H
//	CONTENTS: class HParadoxField
//           class HParadoxRecord
//           class HParadoxTextRecord

#include <fstream.h>
#include "hstring.h"

///////////////////////////////////////////////////////////////////////////
//
// Classes for data fields
//
//      Idendifier          Paradox-type Internal-type
//      ===============
#define FIELD_UNKNOWN '?'  // 
#define FIELD_ALPHA   'A'  // A            HString
#define FIELD_NUMERIC 'N'  // N            double
#define FIELD_SHORT   'S'  // S            short
#define FIELD_MONEY   '$'  // $            double
#define FIELD_DATE    'D'  // D            char[3]
//
///////////////////////////////////////////////////////////////////////////

class HParadoxField
{
public:
	unsigned int isNull : 1;
	char type;

	HParadoxField (unsigned short _type=FIELD_UNKNOWN);
	void Read (const char* text);	// reads based on 'type'.
	operator const char* ();
	void Write (ostream&);
	void SetNullOutFmt (short showNull=0) { nullOutFmt=showNull?1:0; }

	double GetMoney   () const { return v.dVal; }
	double GetNumeric () const { return v.dVal; }
	short  GetShort   () const { return v.sVal; }
	short  GetMonth   () const { return (short)v.dtVal[0]; }
	short  GetDay     () const { return (short)v.dtVal[1]; }
	short  GetYear    () const;	// 1951 - 2050
	
	void   SetString  (const char* text) { str = text; }
	void   SetMoney   (double val) { v.dVal=val; }
	void   SetNumeric (double val) { v.dVal=val; }
	void   SetShort   (short  val) { v.sVal=val; }
	void   SetMonth   (short  val) { v.dtVal[0]=(char)val; }
	void   SetDay     (short  val) { v.dtVal[1]=(char)val; }
	void   SetYear    (short  val) { v.dtVal[2]=(char)(val<2000?val-1900:val-2000); }

protected:
	HString str;
	union { double dVal; short sVal; char dtVal[3]; } v;
	unsigned int nullOutFmt : 1;	// if true, then null 
};


///////////////////////////////////////////////////////////////////////////
//
// Classes for data records
//
///////////////////////////////////////////////////////////////////////////

class HParadoxRecord
{
public:
	HParadoxRecord (const char* description, short nullOutFmt=0);
		// description is a string of 'A', 'N', 'S', '$', 'D' and '?';
		// '?' is used as placeholder for don't-care fields.
	~HParadoxRecord ();
	unsigned short NumFields () const { return numFlds; }
	HParadoxField* Fields () const { return flds; }
	HParadoxField& operator[] (unsigned short idx) const;
	void Output (ostream&, char delimiter=',') const;
	
protected:
	HParadoxField* flds;
	unsigned short numFlds;
private:
	static HParadoxField nul;
};


///////////////////////////////////////////////////////////////////////////
//
// Class for data source, the text file
//
///////////////////////////////////////////////////////////////////////////

#define MAXFIELDS 100

class HParadoxTextRecord : public ifstream
{
public:
	HParadoxTextRecord () { init(); }
	HParadoxTextRecord (const char* path) : ifstream(path) { init(); }
	~HParadoxTextRecord ();
	
	short GetNextRec (int bufsize, HParadoxField*, unsigned short fields);
		// combines getNextLine() and getRecord().
	                                                  
	char  delimiter;
	
private:
	char* lineBuf;
	int   bufSize;
	
	// These are set by each call to GetNextLine() ..
	short stringPtr[MAXFIELDS];
	char  nullFlag[MAXFIELDS];
	short numStrings;
	
	void init ();
	short getNextLine (int bufsize);
		// returns field number or 0 if error or done.
		// get the line, prepare for string pointers and null flags.
	short getRecord (HParadoxField*, unsigned short fields);
		// returns number of fields read.
	const char* getFieldText (unsigned short field);
		// if is a text (quoted by "; double " for " itself), returns a 
		// 'good' one, i.e., no quoting "-s, and a valid single ".
		// if the field has nothing, return NULL.
		// otherwise, return that text string.
	void prepareStrings ();
	
};  // end of class HParadoxTextRecord.


#endif
